@@ -1,11 +1,13 @@
 import colorsys
 
 import numpy as np
-from PIL import ImageEnhance, ImageOps
+from PIL import ImageEnhance, ImageOps, Image
 
-__all__ = ['enhance_saturation', 'enhance_contrast', 'enhance_brightness', 'enhance_sharpness',
-           'enhance_hue', 'gamma', 'color_invert', 'color_clip', 'equalize', 'rgb_hex',
+__all__ = ['enhance_saturation', 'enhance_brightness', 'enhance_sharpness', 'enhance_contrast_sigmoid',
+           'enhance_contrast_log', 'enhance_contrast_linear', 'enhance_contrast_gamma',
+           'enhance_hue', 'color_invert', 'color_clip', 'equalize', 'rgb_hex',
            'hls_to_rgb', 'rgb_to_hls', 'hsv_to_rgb', 'rgb_to_hsv', 'rgb_to_yiq', 'yiq_to_rgb',
+           'rgb_to_yuv', 'yuv_to_rgb', 'dropout'
           ]
 
 
@@ -36,24 +38,19 @@ def enhance_saturation(image, delta, p=1):
     return ImageEnhance.Color(image).enhance(delta)
 
 
-def enhance_contrast(image, delta, p=1):
-    """Adjust contrast of RGB or grayscale images.
+def enhance_contrast_linear(image, delta, p=1):
+    """Adjust image contrast.
   
     Contrast is adjusted independently for each channel of each image.
-    
-    For each channel, this Ops computes the mean of the image pixels in the
-    channel and then adjusts each component `x` of each pixel to
-    `(x - mean) * delta + mean`.
+    pixel = (x - mean) * delta + mean
     
     Tips:
         1 means pixel value no change.
-        0 means all pixel equal. 
-        a suitable interval is (0, 4].
+        0 means all pixel equal, pixel is image mean.
     Args:
         images: a PIL instance.
         delta: if int, float, a float multiplier for adjusting contrast.
-               if list, tuple, randomly picked in the interval
-               `[delta[0], delta[1])` , value is float multiplier for adjusting contrast.
+               if list, tuple, randomly picked in the interval `[delta[0], delta[1])`.
         p: probability that the image does this. Default value is 1.
     Returns:
         a PIL instance.
@@ -65,18 +62,94 @@ def enhance_contrast(image, delta, p=1):
     return ImageEnhance.Contrast(image).enhance(delta)
 
 
+def enhance_contrast_gamma(image, gamma=1, gain=1.0, p=1):
+    """Perform gamma correction on an image.
+    
+    For gamma greater than 1, the histogram will shift towards left and the output image will be darker than the input image. 
+    For gamma less than 1, the histogram will shift towards right and the output image will be brighter than the input image.
+    
+    pixel = 255*gain *((pixel/255)**gamma)
+    Values in the range gamma=(0.5, 2.0) seem to be sensible.
+    
+    Args:
+        image: a PIL instance.
+        gamma: float, Non negative real number, gamma larger than 1 make the shadows darker, 
+               while gamma smaller than 1 make dark regions lighter.
+               if list or tuple, randomly picked in the interval `[gamma[0], gamma[1])`.
+        gain: float, The constant multiplier.
+              if list or tuple, randomly picked in the interval `[gain[0], gain[1])`.
+        p: probability that the image does this. Default value is 1.
+    Returns:
+        a PIL instance.
+    """
+    if np.random.uniform()>p:
+        return image
+    if isinstance(gamma, (list, tuple)):
+        gamma = np.random.randint(gamma[0], gamma[1])
+    if isinstance(gain, (list, tuple)):
+        gain = np.random.uniform(gain[0], gain[1])
+    return image.point(lambda x:int((255 + 1 - 1e-3) * gain * pow(x / 255.0, gamma)))
+
+
+def enhance_contrast_sigmoid(image, cutoff=0.5, gain=10, p=1):
+    """Perform sigmoid correction on an image.
+    
+    pixel = 255*1/(1+exp(gain*(cutoff-pixel/255)))
+    Values in the range gain=(5, 20) and cutoff=(0.25, 0.75) seem to be sensible.
+    
+    Args:
+        image: a PIL instance.
+        cutoff: float, Cutoff that shifts the sigmoid function in horizontal direction. 
+                Higher values mean that the switch from dark to light pixels happens later. 
+                if list or tuple, randomly picked in the interval `[cutoff[0], cutoff[1])`.
+        gain: float, The constant multiplier.
+              if list or tuple, randomly picked in the interval `[gain[0], gain[1])`.
+        p: probability that the image does this. Default value is 1.
+    Returns:
+        a PIL instance.
+    """
+    if np.random.uniform()>p:
+        return image
+    if isinstance(cutoff, (list, tuple)):
+        cutoff = np.random.uniform(cutoff[0], cutoff[1])
+    if isinstance(gain, (list, tuple)):
+        gain = np.random.randint(gain[0], gain[1])
+    return image.point(lambda x:int(255*1/(1+np.exp(gain*(cutoff-x / 255.0)))))
+
+
+def enhance_contrast_log(image, gain=1, p=1):
+    """Perform log correction on an image.
+    
+    pixel = 255*gain*log_2(1+pixel/255)
+    Values in the range gain=[0.6, 1.4] seem to be sensible.
+    
+    Args:
+        image: a PIL instance.
+        gain: float, The constant multiplier.
+              if list or tuple, randomly picked in the interval `[gain[0], gain[1])`.
+        p: probability that the image does this. Default value is 1.
+    Returns:
+        a PIL instance.
+    """
+    if np.random.uniform()>p:
+        return image
+    if isinstance(gain, (list, tuple)):
+        gain = np.random.randint(gain[0], gain[1])
+    return image.point(lambda x:int(255*gain*np.log2(1+x / 255.0)))
+
+
 def enhance_brightness(image, delta, p=1):
-    """Adjust the brightness of RGB or Grayscale images.
+    """Adjust image brightness.
     
     Tips:
-        delta extreme value in the interval [-1, 1], >1 to white, <-1 to black.
-        a suitable interval is [-0.5, 0.5].
-        0 means pixel value no change.
+        delta >1 to white, <1 to black.
+        a suitable interval is [0.5, 1.5].
+        0 means black.
+        1 means pixel value no change.
     Args:
         image: a PIL instance.
         delta: if int, float, Amount to add to the pixel values.
-               if list, tuple, randomly picked in the interval
-               `[delta[0], delta[1])` to add to the pixel values.
+               if list, tuple, randomly picked in the interval `[delta[0], delta[1])`.
         p: probability that the image does this. Default value is 1.
     Returns:
         a PIL instance.
@@ -117,7 +190,7 @@ def enhance_sharpness(image, delta, p=1):
 
 
 def enhance_hue(image, delta, p=1):
-    """Adjust hue of an image.
+    """Adjust image hue.
     
     The image hue is adjusted by converting the image to HSV and
     cyclically shifting the intensities in the hue channel (H).
@@ -150,40 +223,7 @@ def enhance_hue(image, delta, p=1):
     return Image.merge("HSV", (h, s, v)).convert(input_mode)
 
 
-def gamma(image, gamma=1, gain=1.0, p=1):
-    """Perform gamma correction on an image.
-    
-    For gamma greater than 1, the histogram will shift towards left and the output image will be darker than the input image. 
-    For gamma less than 1, the histogram will shift towards right and the output image will be brighter than the input image.
-    Args:
-        image: a PIL instance.
-        gamma: float, Non negative real number, gamma larger than 1 make the shadows darker, 
-               while gamma smaller than 1 make dark regions lighter.
-               if list or tuple, randomly picked in the interval `[gamma[0], gamma[1])`.
-        gain: float, The constant multiplier.
-              if list or tuple, randomly picked in the interval `[gain[0], gain[1])`.
-        p: probability that the image does this. Default value is 1.
-    Returns:
-        a PIL instance.
-    """
-    if np.random.uniform()>p:
-        return image
-    if isinstance(gamma, (list, tuple)):
-        gamma = np.random.randint(gamma[0], gamma[1])
-    if isinstance(gain, (list, tuple)):
-        gain = np.random.randint(gain[0], gain[1])
-    assert gamma >= 0, "Gamma should be a non-negative real number."
-    input_mode = image.mode
-    if input_mode!="RGB":
-        image = image.convert("RGB")
-    gamma_map = [int((255 + 1 - 1e-3) * gain * pow(ele / 255.0, gamma)) for ele in range(256)] * 3
-    image = image.point(gamma_map)
-    if input_mode!="RGB":
-        image = image.convert(input_mode)
-    return image
-
-
-def color_invert(image, lower=None, upper=None, p=1):
+def color_invert(image, lower=None, upper=None, wise='pixel', prob=1, p=1):
     """Invert colors of input PIL image.
 
     Args:
@@ -192,6 +232,8 @@ def color_invert(image, lower=None, upper=None, p=1):
                if list or tuple, randomly picked in the interval `[lower[0], lower[1])`.
         upper: int or list or tuple, [0, 255], All pixels above this greyscale level are inverted.
                if list or tuple, randomly picked in the interval `[upper[0], upper[1])`.
+        wise: 'pixel' or 'channel' or list of channel, method of applying operate.
+        prob: probability of every pixel or channel being changed.
         p: probability that the image does this. Default value is 1.
     Returns:
         a PIL instance.
@@ -208,10 +250,25 @@ def color_invert(image, lower=None, upper=None, p=1):
         lower = 0
     elif upper is None:
         upper = 255
-    return image.point(lambda x: 255-x if x<=lower or x>=upper else x)
+    if wise=='pixel':
+        return image.point(lambda x: 255-x if (x<=lower or x>=upper) and np.random.uniform()<prob else x)
+    elif wise=='channel':
+        split = list(image.split())
+        for i in range(len(split)):
+            if np.random.uniform()<prob:
+                split[i] = split[i].point(lambda x: 255-x if x<=lower or x>=upper else x)
+        return Image.merge(image.mode, split)
+    elif isinstance(wise, (list, tuple)):
+        split = list(image.split())
+        for i in wise:
+            if np.random.uniform()<prob:
+                split[i] = split[i].point(lambda x: 255-x if x<=lower or x>=upper else x)
+        return Image.merge(image.mode, split)
+    else:
+        raise ValueError("`wise` should be 'pixel' or 'channel' or list of channel.")
 
 
-def color_clip(image, lower=None, upper=None, p=1):
+def color_clip(image, lower=None, upper=None, wise='pixel', prob=1, p=1):
     """Clip colors of input PIL image.
     
     Args:
@@ -220,6 +277,8 @@ def color_clip(image, lower=None, upper=None, p=1):
                if list or tuple, randomly picked in the interval `[lower[0], lower[1])`.
         upper: int or list or tuple, [0, 255], All pixels above this greyscale level are clipped.
                if list or tuple, randomly picked in the interval `[upper[0], upper[1])`.
+        wise: 'pixel' or 'channel' or list of channel, method of applying operate.
+        prob: probability of every pixel or channel being changed.
         p: probability that the image does this. Default value is 1.
     Returns:
         a PIL instance.
@@ -232,11 +291,32 @@ def color_clip(image, lower=None, upper=None, p=1):
         upper = np.random.randint(upper[0], upper[1])
     if lower is None and upper is None:
         raise ValueError('`lower` and `upper` at least one value.')
-    if lower is not None:
-        image = image.point(lambda x: lower if x<=lower else x)
-    if upper is not None:
-        image = image.point(lambda x: upper if x>=upper else x)
-    return image
+    if wise=='pixel':
+        if lower is not None:
+            image = image.point(lambda x: lower if x<=lower and np.random.uniform()<prob else x)
+        if upper is not None:
+            image = image.point(lambda x: upper if x>=upper and np.random.uniform()<prob else x)
+        return image
+    elif wise=='channel':
+        split = list(image.split())
+        for i in range(len(split)):
+            if np.random.uniform()<prob:
+                if lower is not None:
+                    split[i] = split[i].point(lambda x: lower if x<=lower else x)
+                if upper is not None:
+                    split[i] = split[i].point(lambda x: upper if x>=upper else x)
+        return Image.merge(image.mode, split)
+    elif isinstance(wise, (list, tuple)):
+        split = list(image.split())
+        for i in wise:
+            if np.random.uniform()<prob:
+                if lower is not None:
+                    split[i] = split[i].point(lambda x: lower if x<=lower else x)
+                if upper is not None:
+                    split[i] = split[i].point(lambda x: upper if x>=upper else x)
+        return Image.merge(image.mode, split)
+    else:
+        raise ValueError("`wise` should be 'pixel' or 'channel' or list of channel.")
 
 
 def equalize(image, p=1):
@@ -355,6 +435,39 @@ def yiq_to_rgb(image, normalize=False, dtype='float32'):
     return (np.stack((r, g, b), axis=2)*norm).astype(dtype)
 
 
+def rgb_to_yuv(image):
+    """Converts image from RGB to YUV.
+
+    Args:
+        image: NumPy RGB image array of shape (H, W, C) to be converted.
+    Returns:
+        a numpy array.
+    """
+    if image.max()>1:
+        image = image/255.
+    kernel = np.array([[0.299, -0.14714119, 0.61497538],
+                       [0.587, -0.28886916, -0.51496512],
+                       [0.114, 0.43601035, -0.10001026]])
+    return np.dot(image, kernel)
+
+
+def yuv_to_rgb(image, normalize=False, dtype='float32'):
+    """Converts image from YUV to RGB.
+
+    Args:
+        image: NumPy YUV image array of shape (H, W, C) to be converted.
+        normalize: if True, rgb numpy array is [0,255], if False, rgb numpy array is [0,1]
+        dtype: rgb numpy array dtype.
+    Returns:
+        a numpy array.
+    """
+    norm = 255. if normalize else 1.
+    kernel = np.array([[1, 1, 1], 
+                       [0, -0.394642334, 2.03206185],
+                       [1.13988303, -0.58062185, 0]])
+    return (np.dot(image, kernel)*norm).astype(dtype)
+
+
 def rgb_hex(color):
     """rgb to hexadecimal or hexadecimal to rgb.
     
@@ -368,4 +481,47 @@ def rgb_hex(color):
             return (int(color[1:3], 16), int(color[3:5], 16), int(color[5:7], 16))
     if isinstance(color, (list, tuple)):
         return '#'+''.join([str(hex(i))[-2:].replace('x', '0').upper() for i in color])
-    raise ValueError('`color` value error, should be str or list or tuple.')
\ No newline at end of file
+    raise ValueError('`color` value error, should be str or list or tuple.')
+    
+
+def dropout(image, value=0, wise='pixel', prob=0.1, p=1):
+    """Drop random channels from images.
+
+    For image data, dropped channels will be filled with value.
+    
+    Args:
+        image: a PIL instance.
+        value: int or list, dropped channels will be filled with value.
+        wise: 'pixel' or 'channel' or list of channel, method of applying operate.
+        prob: probability of every pixel or channel being changed.
+        p: probability that the image does this. Default value is 1.
+    returns: 
+        a PIL instance.
+    """
+    if np.random.uniform()>p:
+        return image
+    if isinstance(value, (int, float)):
+        value = [int(value)]
+    if wise=='pixel':
+        return image.point(lambda x:np.random.choice(value) if np.random.uniform()<prob else x)
+    elif wise=='channel':
+        split = list(image.split())
+        index = np.random.randint(0, len(split))
+        for i in range(len(split)):
+            if np.random.uniform()<prob and i!=index:
+                split[i] = split[i].point(lambda x:np.random.choice(value))
+        return Image.merge(image.mode, split)
+    elif isinstance(wise, (list, tuple)):
+        split = list(image.split())
+        index = np.random.randint(0, len(split))
+        if index in wise:
+            if len(split)==len(wise):
+                index = 0
+            else:
+                index = -1
+        for i in wise:
+            if np.random.uniform()<prob and i!=index:
+                split[i] = split[i].point(lambda x:np.random.choice(value))
+        return Image.merge(image.mode, split)
+    else:
+        raise ValueError("`wise` value error.")
\ No newline at end of file
