@@ -66,6 +66,8 @@ class RandomSearch():
             params error.
         """
         logger = Logger(name='xgb')
+        logger.info(f"api is deprecated and will be removed in 1.5.0")
+        logger.info(f"please use la.param_search.RandomSearch")
         import warnings
         warnings.filterwarnings("ignore")
         import xgboost as xgb
