@@ -82,7 +82,7 @@ class HyperParametersRandom():
             self.space[name] = {'mode':'Dependence', 'dependent_name':dependent_name, 'function':function, 'default':default}
             self.params[name] = function(self.params[dependent_name]) if default is None else default
     
-    def update(self):
+    def update(self, best_params=None):
         """params update"""
         if self._update_init:
             self._update_init = False
@@ -100,7 +100,18 @@ class HyperParametersRandom():
                     self.params[name] = config['function'](self.params[config['dependent_name']])
         self.params_history[self._update_nums] = self.params.copy()
         self._update_nums += 1
+        if best_params is not None:
+            self.best_params = best_params.copy()
 
+    def from_HyperParameters(self, hp):
+        """update HyperParametersRandom class.
+        
+        Args:
+            hp: a HyperParametersRandom class.
+        """
+        for name, config in hp.space.items():
+            self.space[name] = config
+            self.params[name] = hp.params[name]
 
 class HyperParametersGrid():
     def __init__(self):
@@ -121,11 +132,12 @@ class HyperParametersGrid():
         Args:
             name: Str. Name of parameter. Must be unique.
             default: Default value to return for the parameter. If unspecified, the default value will be None.
-            rank: Int, default 0, Importance ordering of parameters, smaller is more important.
+            rank: Int, default 0, Importance ordering of parameters, smaller is more important, rank should be greater than 1.
         """
         if name not in self.space:
             self.space[name] = {'mode':'Boolean', 'default':default, 'rank':rank, 'values':[True, False]}
-            self._rank[rank].append(name)
+            if rank>0:
+                self._rank[rank].append(name)
             self.params[name] = np.random.choice([True, False]) if default is None else default
         
     def Float(self, name, min_value, max_value, rounds=1, default=None, rank=0):
@@ -137,12 +149,13 @@ class HyperParametersGrid():
             max_value: Float. Upper bound of the range.
             rounds: Optional. Int, e.g. 2 mean round(x, 2). smallest meaningful distance between two values. 
             default: Default value to return for the parameter. If unspecified, the default value will be None.
-            rank: Int, default 0, Importance ordering of parameters, smaller is more important.
+            rank: Int, default 0, Importance ordering of parameters, smaller is more important, rank should be greater than 1.
         """
         if name not in self.space:
             self.space[name] = {'mode':'Float', 'default':default, 'rank':rank, 
                                 'values':np.linspace(min_value, max_value, round((max_value-min_value)*10**rounds)+1).round(rounds).tolist()}
-            self._rank[rank].append(name)
+            if rank>0:
+                self._rank[rank].append(name)
             self.params[name] = round(random.uniform(min_value, max_value), rounds) if default is None else default
         
     def Int(self, name, min_value, max_value, default=None, rank=0):
@@ -154,11 +167,12 @@ class HyperParametersGrid():
             min_value: Int. Lower limit of range (included).
             max_value: Int. Upper limit of range (included).
             default: Default value to return for the parameter. If unspecified, the default value will be None.
-            rank: Int, default 0, Importance ordering of parameters, smaller is more important.
+            rank: Int, default 0, Importance ordering of parameters, smaller is more important, rank should be greater than 1.
         """
         if name not in self.space:
             self.space[name] = {'mode':'Int', 'default':default, 'rank':rank, 'values':list(range(min_value, max_value+1))}
-            self._rank[rank].append(name)
+            if rank>0:
+                self._rank[rank].append(name)
             self.params[name] = round(random.uniform(min_value, max_value)) if default is None else default
         
     def Choice(self, name, values, default=None, rank=0):
@@ -169,11 +183,12 @@ class HyperParametersGrid():
             name: Str. Name of parameter. Must be unique.
             values: List of possible values. Values must be int, float, str, or bool. All values must be of the same type.
             default: Default value to return for the parameter. If unspecified, the default value will be None.
-            rank: Int, default 0, Importance ordering of parameters, smaller is more important.
+            rank: Int, default 0, Importance ordering of parameters, smaller is more important, rank should be greater than 1.
         """
         if name not in self.space:
             self.space[name] = {'mode':'Choice', 'default':default, 'rank':rank, 'values':list(values)}
-            self._rank[rank].append(name)
+            if rank>0:
+                self._rank[rank].append(name)
             self.params[name] = random.choice(self.space[name]['values']) if default is None else default
         
     def Dependence(self, name, dependent_name, function, default=None):
@@ -216,6 +231,7 @@ class HyperParametersGrid():
             self._update_init = False
         else:
             if best_params is not None:
+                self.best_params = best_params.copy()
                 self.params = best_params.copy()
             self.params.update(self._rank_list[self._update_nums-1])
         self.params_history[self._update_nums] = self.params.copy()
@@ -224,4 +240,30 @@ class HyperParametersGrid():
     def cardinality(self):
         if not self._rank_list:
             self._rank_list_func()
-        return self._cardinality
\ No newline at end of file
+        return self._cardinality
+    
+    def grid_space(self):
+        return self._rank_list
+    
+    def from_HyperParameters(self, hp):
+        """update HyperParametersGrid class.
+        
+        Args:
+            hp: a HyperParametersGrid class.
+        """
+        for name, config in hp.space.items():
+            self.space[name] = config
+            self.params[name] = hp.params[name]
+        for rank, name_list in hp._rank.items():
+            if rank not in self._rank:
+                self._rank[rank].append(name_list)
+                continue
+            for name in name_list:
+                for r, n in self._rank.items():
+                    if rank==r:
+                        if name not in self._rank[rank]:
+                            self._rank[rank].append(name)
+                    else:
+                        if name in self._rank[rank]:
+                            self._rank[rank].remove(name)
+                        
\ No newline at end of file
