@@ -6,6 +6,7 @@ import numpy as np
 __all__ = ['select_best_length', 'word_to_index', 'sequence_word_index', 'index_vector_matrix',
            'sequence_index_word']
 
+
 def select_best_length(sequence, sample_rate=0.8):
     """Select best length for sequence with keep rate.
     
@@ -18,18 +19,20 @@ def select_best_length(sequence, sample_rate=0.8):
     t = sorted(map(lambda x:len(x), sequence))
     return t[int(np.ceil(len(sequence)*sample_rate))-1]
 
+
 def word_to_index(sequence):
     """Sequence word transfer to index.
     
     Args:
         sequence: pd.Series or np.array or List of lists, sample sequence.
     Returns:
-        dict, {word: index}, The higher the frequency, the higher the ranking.
+        dict, {'positive':{word: index}，'negative':{index:word}}, The higher the frequency, the higher the ranking.
     """
     t = Counter(chain.from_iterable(sequence))
     t = sorted(t, key=t.get, reverse=True)
     return {'positive':{v: k + 1 for k, v in enumerate(t)}, 'negative':{k + 1:v for k, v in enumerate(t)}}
 
+
 def sequence_word_index(sequence, word_index_dict, pad_value=0):
     """Sequence word transfer to sequence index.
     
@@ -44,25 +47,31 @@ def sequence_word_index(sequence, word_index_dict, pad_value=0):
         return [word_index_dict.get(j, pad_value) for j in sequence]
     return [[word_index_dict.get(j, pad_value) for j in i] for i in sequence]
 
-def sequence_index_word(sequence, index_word_dict, pad_value=' ', join=False):
+
+def sequence_index_word(sequence, index_word_dict, pad_value=' ', join=None):
     """Sequence index transfer to sequence word.
     
     Args:
         sequence: pd.Series or np.array or List of lists, sample sequence.
         index_word_dict: dict, {index: word}.
         pad_value: fillna value, if word not in index_word_dict, fillna it.
-        join: whether to merge the converted words.
+        join: str, whether to merge the converted words. represents the merge interval
     Returns:
         List of lists, sequence index transfer to sequence word list.
     """
     if isinstance(sequence[0], int):
-        if join:
+        if join=='':
             return ''.join([index_word_dict.get(j, pad_value) for j in sequence])
+        if join:
+            return f'{join}'.join([index_word_dict.get(j, pad_value) for j in sequence])
         return [index_word_dict.get(j, pad_value) for j in sequence]
+    if join=='':
+            return [''.join([index_word_dict.get(j, pad_value) for j in i]) for i in sequence]
     if join:
-        return [''.join([index_word_dict.get(j, pad_value) for j in i]) for i in sequence]
+        return [f'{join}'.join([index_word_dict.get(j, pad_value) for j in i]) for i in sequence]
     return [[index_word_dict.get(j, pad_value) for j in i] for i in sequence]
 
+
 def index_vector_matrix(word_index_dict, word_vector_dict, embed_dim=300, initialize='norm', dtype='float32'):
     """Make index vector matrix with shape `(len(word_index_dict), embed_dim)`.
     
