@@ -1,14 +1,11 @@
 import time
 
+from linora.utils._config import Config
 from linora.utils._logger import Logger
 
 __all__ = ['Progbar']
 
 
-class Param:
-    pass
-
-
 class Progbar():
     """Displays a progress bar."""
     def __init__(self, target, width=25, verbose=1, unit_name='step'):
@@ -19,7 +16,7 @@ class Progbar():
             verbose: Verbosity mode, 0 (silent), 1 (verbose)
             unit_name: Display name for step counts (usually "step" or "sample").
         """
-        self.param = Param
+        self.param = Config()
         self.param.width = width
         self.param.target = target
         self.param.time = time.time()
@@ -36,6 +33,8 @@ class Progbar():
             current: add Index of current step, current += current.
             values: List of tuples: (name, value_for_last_step). 
         """
+        if not self.param.verbose:
+            return 0
         self.param.n += 1
         self.param.current += current
         if self.param.target is not None:
@@ -58,13 +57,13 @@ class Progbar():
             msg = msg+f"- {int(time_diff/self.param.n*1000)}ms/{self.param.unit_name}"
         if values is not None:
             msg = msg+' - '+''.join([f"{i[0]}: {i[1]} " for i in values])
-        if self.param.verbose:
-            if self.param.target is None:
-                self.param.logger.info(msg+' '*4, enter=False)
-            elif self.param.current<self.param.target:
-                self.param.logger.info(msg+' '*4, enter=False)
-            else:
-                self.param.logger.info(msg+' '*4, enter=True)
+        
+        if self.param.target is None:
+            self.param.logger.info(msg+' '*4, enter=False)
+        elif self.param.current<self.param.target:
+            self.param.logger.info(msg+' '*4, enter=False)
+        else:
+            self.param.logger.info(msg+' '*4, enter=True)
             
     def update(self, current, values=None):
         """
@@ -72,6 +71,8 @@ class Progbar():
             current: update Index of current step.
             values: List of tuples: (name, value_for_last_step). 
         """
+        if not self.param.verbose:
+            return 0
         self.param.n += 1
         if self.param.target is not None:
             if current>self.param.target:
@@ -93,10 +94,10 @@ class Progbar():
             msg = msg+f"- {int(time_diff/self.param.n*1000)}ms/{self.param.unit_name}"
         if values is not None:
             msg = msg+' - '+''.join([f"{i[0]}: {i[1]} " for i in values])
-        if self.param.verbose:
-            if self.param.target is None:
-                self.param.logger.info(msg+' '*4, enter=False)
-            elif current<self.param.target:
-                self.param.logger.info(msg+' '*4, enter=False)
-            else:
-                self.param.logger.info(msg+' '*4, enter=True)
+        
+        if self.param.target is None:
+            self.param.logger.info(msg+' '*4, enter=False)
+        elif current<self.param.target:
+            self.param.logger.info(msg+' '*4, enter=False)
+        else:
+            self.param.logger.info(msg+' '*4, enter=True)
